@@ -5,16 +5,14 @@ from __future__ import with_statement
 import re
 import os.path
 import datetime
-import tempfile
 import subprocess
 import HTMLParser
 
 
 class SqlplusCommando(object):
 
-    QUERY_ERROR_HANDLER = '''WHENEVER SQLERROR EXIT SQL.SQLCODE;
-WHENEVER OSERROR EXIT 9;
-'''
+    CATCH_ERRORS = "WHENEVER SQLERROR EXIT SQL.SQLCODE;\nWHENEVER OSERROR EXIT 9;\n"
+    EXIT_COMMAND = "\ncommit;\nexit;\n"
     ISO_FORMAT = '%Y-%m-%d %H:%M:%S'
 
     def __init__(self, configuration=None,
@@ -32,52 +30,40 @@ WHENEVER OSERROR EXIT 9;
             self.username = configuration['username']
             self.password = configuration['password']
         else:
-            raise Exception('Missing database configuration')
+            raise SqlplusException('Missing database configuration')
         self.cast = cast
 
     def run_query(self, query, parameters={}, cast=True,
                   check_unknown_command=True):
-        query = self._process_parameters(query, parameters)
-        command = self.QUERY_ERROR_HANDLER + query
-        return self._run_command(command, cast=cast,
-            check_unknown_command=check_unknown_command)
-
-    def run_script(self, script, cast=True, check_unknown_command=True):
-        if not os.path.isfile(script):
-            raise Exception("Script '%s' was not found" % script)
-        with open(script) as stream:
-            source = stream.read()
-        filename = tempfile.mkstemp(prefix='sqlplus_commando-',
-                                    suffix='.sql')[1]
-        with open(filename, 'wb') as stream:
-            stream.write(self.QUERY_ERROR_HANDLER + source)
-        try:
-            return self._run_command("@%s" % filename, cast=cast,
-                check_unknown_command=check_unknown_command)
-        finally:
-            os.remove(filename)
-
-    def _run_command(self, command, cast, check_unknown_command):
-        connection_url = self._get_connection_url()
+        if parameters:
+            query = self._process_parameters(query, parameters)
+        query = self.CATCH_ERRORS + query
         session = subprocess.Popen(['sqlplus', '-S', '-L', '-M', 'HTML ON',
-                                    connection_url],
+                                    self._get_connection_url()],
                                    stdin=subprocess.PIPE,
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE)
-        session.stdin.write(command)
-        output, _ = session.communicate()
+        session.stdin.write(query)
+        output, _ = session.communicate(self.EXIT_COMMAND)
         code = session.returncode
         if code != 0:
-            raise Exception(OracleErrorParser.parse(output))
+            raise SqlplusException(SqlplusErrorParser.parse(output), query)
         else:
             if output:
-                result = OracleResponseParser.parse(output, cast=cast,
-                                                    check_unknown_command=check_unknown_command)
+                result = SqlplusResultParser.parse(output, cast=cast,
+                                                  check_unknown_command=check_unknown_command)
                 return result
 
+    def run_script(self, script, cast=True, check_unknown_command=True):
+        if not os.path.isfile(script):
+            raise SqlplusException("Script '%s' was not found" % script)
+        with open(script) as stream:
+            source = stream.read()
+        return self.run_query(query=source, cast=cast, check_unknown_command=check_unknown_command)
+
     def _get_connection_url(self):
         return "%s/%s@%s/%s" % \
-            (self.username, self.password, self.hostname, self.database)
+               (self.username, self.password, self.hostname, self.database)
 
     @staticmethod
     def _process_parameters(query, parameters):
@@ -109,15 +95,15 @@ WHENEVER OSERROR EXIT 9;
         elif parameter is None:
             return "NULL"
         else:
-            raise Exception("Type '%s' is not managed as a query parameter" %
-                            parameter.__class__.__name__)
+            raise SqlplusException("Type '%s' is not managed as a query parameter" %
+                                   parameter.__class__.__name__)
 
     @staticmethod
     def _escape_string(string):
         return string.replace("'", "''")
 
 
-class OracleResponseParser(HTMLParser.HTMLParser):
+class SqlplusResultParser(HTMLParser.HTMLParser):
 
     DATE_FORMAT = '%d/%m/%y %H:%M:%S'
     UNKNOWN_COMMAND = 'SP2-0734: unknown command'
@@ -126,7 +112,7 @@ class OracleResponseParser(HTMLParser.HTMLParser):
         (r'-?\d*,?\d*([Ee][+-]?\d+)?', lambda f: float(f.replace(',', '.'))),
         (r'\d\d/\d\d/\d\d \d\d:\d\d:\d\d,\d*',
          lambda d: datetime.datetime.strptime(d[:17],
-                                              OracleResponseParser.DATE_FORMAT)),
+                                              SqlplusResultParser.DATE_FORMAT)),
         (r'NULL', lambda d: None),
     )
 
@@ -142,9 +128,11 @@ class OracleResponseParser(HTMLParser.HTMLParser):
 
     @staticmethod
     def parse(source, cast, check_unknown_command):
-        if OracleResponseParser.UNKNOWN_COMMAND in source and check_unknown_command:
-            raise Exception(OracleErrorParser.parse(source))
-        parser = OracleResponseParser(cast)
+        if not source.strip():
+            return ()
+        if SqlplusResultParser.UNKNOWN_COMMAND in source and check_unknown_command:
+            raise SqlplusException(SqlplusErrorParser.parse(source))
+        parser = SqlplusResultParser(cast)
         parser.feed(source)
         return tuple(parser.result)
 
@@ -181,13 +169,13 @@ class OracleResponseParser(HTMLParser.HTMLParser):
 
     @staticmethod
     def _cast(value):
-        for regexp, function in OracleResponseParser.CASTS:
+        for regexp, function in SqlplusResultParser.CASTS:
             if re.match("^%s$" % regexp, value):
                 return function(value)
         return value
 
 
-class OracleErrorParser(HTMLParser.HTMLParser):
+class SqlplusErrorParser(HTMLParser.HTMLParser):
 
     UNKNOWN_COMMAND = 'SP2-0734: unknown command'
 
@@ -198,7 +186,7 @@ class OracleErrorParser(HTMLParser.HTMLParser):
 
     @staticmethod
     def parse(source):
-        parser = OracleErrorParser()
+        parser = SqlplusErrorParser()
         parser.feed(source)
         return '\n'.join([l for l in parser.message.split('\n') if l.strip() != ''])
 
@@ -213,3 +201,13 @@ class OracleErrorParser(HTMLParser.HTMLParser):
     def handle_data(self, data):
         if self.active:
             self.message += data
+
+# pylint: disable=W0231
+class SqlplusException(Exception):
+
+    def __init__(self, message, query=None):
+        self.message = message
+        self.query = query
+
+    def __str__(self):
+        return self.message
